#this is the file of the measurements for the stress and sampling. It contains setup,stress,sampling

#import sys
#sys.path.insert(0, '..') #append parent directory

import hp4155a 

def setup(device):
    device.reset()
    
    #define user functions
    device.user_function('t','s','@TIME')
    device.user_function('IDmm','mA/mm','1E4*ID')
    device.user_function('IGmm','mA/mm', '1E4*IG')
    device.user_function('ABSIGm','mA/mm','ABS(IGmm)')
    device.user_function('ABSIDm','mA/mm','ABS(IDmm)')

    #disable vmus and vsus for sterss and sampling
    device.stress_disable_not_smu() #stress
    device.disable_not_smu() #sampling


def sampling(VDS,VGS,VDS_comp,VGS_comp,sampling_mode,number_of_points,integration_time,initial_interval,hold_time,filter_status,device):
    device.measurement_mode('SAMP')

    smu1 = {
        "vname": "VS1",
        "iname": "IS1",
        "mode": "COMM",
        "func": "CONS"
    }

    smu2 = {
        "vname": "VDS",
        "iname": "ID",
        "mode": "V",
        "func": "CONS",
        "value": VDS,
        "comp": VDS_comp
        
    }

    smu3 = {
        "vname" : "VGS",
        "iname" : "IG",
        "mode" : "V",
        "func" : "CONS",
        "value": VGS,
        "comp": VGS_comp
    }

    smu4 = {
        "vname": "VS2",
        "iname": "IS2",
        "mode": "COMM",
        "func": "CONS"
    }
            
    #setup of smus for sampling
    device.setup_smu(1,smu1)
    device.setup_smu(2,smu2)
    device.setup_smu(3,smu3)
    device.setup_smu(4,smu4)
    

    #set the constant value of smus and compilance
    device.setup_smu_sampling(2,smu2)
    device.setup_smu_sampling(3,smu3)
   
    #set sampling parameters
    parameters = device.sampling_parameters_dict()
    parameters['mode'] = sampling_mode

    #the maximum number of samples for logarithmic mode is 11 decades +1 although it is allowed 10001 samples
    if sampling_mode == "L10" and number_of_points>111:
        parameters['points'] = 111
    elif sampling_mode == "L25" and number_of_points>276:
        parameters['points'] = 276
    elif sampling_mode=="L50" and number_of_points>551:
        parameters['points'] = 551
    else:   
        parameters["points"] = number_of_points
    

    parameters["interval"] = initial_interval
    parameters["hold"] = hold_time
    parameters["filter"] = filter_status

    device.setup_sampling(parameters)
    device.integration_time(integration_time)

    #display variables
    device.display_variable('X','t')
    device.display_variable('Y1','ABSIDm')
    device.display_variable('Y2','ABSIGm')

    #start measurement
    device.single_measurement()
    while device.operation_completed() == False:
        pass
    device.autoscaling()

    time = device.return_values('t')
    IDmm = device.return_values('IDmm')
    IGmm = device.return_values('IGmm')
    return time,IDmm,IGmm

def stress(VDS,VGS,duration,comp,device):
    device.stress_page() #go to stress page
    #define smus to numbers in mode and values for stress

    smu1 = {
        "mode":"COMM",
        "func":"NSYNC",
        "name": "VS1"
    }

    smu2 = {
        "mode":"V",
        "func":"SYNC",
        "name": "VD",
        "value": VDS,
        "comp": comp
    }

    smu3 = {
        "mode":"V",
        "func":"SYNC",
        "name": "VG",
        "value": VGS,
        "comp": comp
    }

    smu4 = {
        "mode":"COMM",
        "func":"NSYNC",
        "name": "VS2"
    }

    device.smu_stress(1,smu1)
    device.smu_stress(2,smu2)
    device.smu_stress(3,smu3)
    device.smu_stress(4,smu4)
    #values
    device.setup_smu_stress(2,smu2)
    device.setup_smu_stress(3,smu3)
    #time
    device.stress_time(duration)

    device.start_stress()
    while device.operation_completed() == False:
        pass
    return